import java.io.*;
import java.util.*;
import javax.xml.parsers.*;

/**
 * A console application to reverse the masking of a corpus XML file using a ConversionDictionary stored in a file
 * @author Johannes Dellert <jdellert@sfs.uni-tuebingen.de>
 * @version 0.1, November 2007
 */

public class CorpusDemasker
{

    /**
     * This program serves to reverse the masking of a corpus XML file using a ConversionDictionary stored in a file
     * Usage:   CorpusDemasker (--userDefPaths) [key file] [XML-File] [output file]
     *          Option --userDefPaths to manually specify extracted elements
     */
    public static void main(String[] args)
    {
        //we always need three files: input file (xmlFile) and output file as well as the dictionary (keyFile
        File xmlFile = null;
        File outputFile = null;
        File keyFile = null;

        //these strings specify where in the input document the decisive information is to be found
        String orthPath = "../orth";
        String posPath = "../pos";
        //are locations defined by the user?
        boolean userDefPaths = false;

        //check for correct argument structure
        if (args.length < 3)
        {
            printUsage();
            System.exit(1);
        }
        if (args.length == 3)
        {
            keyFile = new File(args[0]);
            xmlFile = new File(args[1]);
            outputFile = new File(args[2]);
        }
        if (args.length == 4)
        {
            // enable user defined tags to ensure compatibility with many corpora
            if (args[0].equals("--userDefPaths")) 
            {
                userDefPaths = true;
                keyFile = new File(args[1]);
                xmlFile = new File(args[2]);
                outputFile = new File(args[3]);
            }
            else 
            {
                printUsage();
                System.exit(1);
            }
        }
        //let the user define the tags if desired
        if (userDefPaths)
        {
            System.out.print("Please specify the XPath to the elements containing the words: ");
            Scanner tagNameScanner = new Scanner(System.in);
            orthPath = tagNameScanner.next(); 
            System.out.print("Please specify the XPath to the elements containing the POS tags: ");
            tagNameScanner = new Scanner(System.in);
            posPath = tagNameScanner.next(); 
        }
        //the replacer will be given a reverse dictionary (mode 3 makes it use the dictionary)
        Replacer replacer = new Replacer(outputFile, 3, orthPath, posPath, null);
        boolean loadSuccess = xmlFile.canRead();
        if (loadSuccess)
        {
            try
            {
                SAXParserFactory factory = SAXParserFactory.newInstance();
                SAXParser saxParser = factory.newSAXParser();
                //let ConvDict read out the key file (in reverse) and hand it on to the replacer
                replacer.setConvDict(ConversionDictionary.readReverse(keyFile));
                //now let the replacer process the orth elements by means of the dictionary
                saxParser.parse(xmlFile, replacer);
            }
            catch (Throwable t)
            {
                t.printStackTrace();
            }
        }
        else
        {
            System.out.println("File does not exist!");
        }
    }

    private static void printUsage()
    {
        System.out.println("Usage:\t CorpusDemasker (--userDefPaths) [key file] [XML-File] [output file]");
    }
} 