import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.*;

import java.io.*;

/**
 * A GUI frontend to mask a corpus XML file using a wide range of different methods
 * @author Johannes Dellert <jdellert@sfs.uni-tuebingen.de>
 * @version 0.1, November 2007
 */

public class CorpusMaskerGUI extends JFrame implements ActionListener, ChangeListener
{
    //to keep eclipse and the compiler happy
    static final long serialVersionUID = 1;
    
    //parameters for masking
    int maskingMode;

    String orthPath;
    String posPath;

    Set<String> preservedPOS;
    Set<String> noMorphPOS;

    double occPerWord;
    int minOccurs;

    //elements of the GUI
    JTextPane fileSample;
    JTextPane previewSample;

    JButton startButton;
    JButton cancelButton;

    JLabel errorMessages;

    JTextArea logText;

    JTextField inputFileChooser;
    JTextField orthPathChooser;
    JTextField posPathChooser;
    JTextField outputFileChooser;
    JTextField dictFileChooser;
    JTextField ccfFileChooser;
    JTextField preservedPOSChooser;
    JTextField noMorphPOSChooser;
    JTextField occPerWordChooser;
    JTextField minOccursChooser;

    JButton inputFileChanger;
    JButton orthPathChanger;
    JButton posPathChanger;
    JButton outputFileChanger;
    JButton dictFileChanger;
    JButton ccfFileChanger;
    JButton preservedPOSChanger;
    JButton noMorphPOSChanger;
    JButton occPerWordChanger;
    JButton minOccursChanger;

    JLabel inputFileLabel;
    JLabel orthPathLabel;
    JLabel posPathLabel;
    JLabel outputFileLabel;
    JLabel maskingModeLabel;
    JLabel dictFileLabel;
    JLabel ccfFileLabel;
    JLabel preservedPOSLabel;
    JLabel noMorphPOSLabel;
    JLabel occPerWordLabel;
    JLabel minOccursLabel;

    JSlider levelChanger;

    //input and output parameters
    String inputFileName;
    String outputFileName;
    String dictFileName;
    String ccfFileName;

    File inputFile;
    File outputFile;
    File dictFile;
    
    //threads to prevent frontend from freezing during the masking process
    Thread guiThread;
    Thread errorThread;
    Thread workerThread;

    /**
     * This GUI window consists of two text panes for original and masking preview at the sides
     * and the controller components plus a logging window in the center
     */
    public CorpusMaskerGUI()
    {
        //default behaviour: strip everything
        maskingMode = 4;

        //default masking parameters as applicable to the TubaDZ corpus in TUSNELDA format
        orthPath = "../orth";
        posPath = "../pos";
        
        ccfFileName = "lang/german.ccf";

        preservedPOS = new HashSet<String>();
        noMorphPOS = new HashSet<String>();

        occPerWord = 0.02;
        minOccurs = 1;

        //layout specification
        Border blackline;
        blackline = BorderFactory.createLineBorder(Color.black);

        fileSample = new JTextPane();
        fileSample.setBorder(blackline);
        fileSample.setSize(new Dimension(300, 500));
        fileSample.setMinimumSize(new Dimension(300, 500));
        fileSample.setMaximumSize(new Dimension(300, 500));
        fileSample.setPreferredSize(new Dimension(300, 500));
        fileSample.setContentType("text/html");
        fileSample.setEditable(false);
        previewSample = new JTextPane();
        previewSample.setBorder(blackline);
        previewSample.setSize(new Dimension(300, 500));
        previewSample.setMinimumSize(new Dimension(300, 500));
        previewSample.setMaximumSize(new Dimension(300, 500));
        previewSample.setPreferredSize(new Dimension(300, 500));
        previewSample.setContentType("text/html");
        previewSample.setEditable(false);
        
        JScrollPane fileSampleScrollPane = new JScrollPane(fileSample);
        fileSampleScrollPane.setSize(new Dimension(300, 500));
        fileSampleScrollPane.setMinimumSize(new Dimension(300, 500));
        fileSampleScrollPane.setMaximumSize(new Dimension(300, 500));
        fileSampleScrollPane.setPreferredSize(new Dimension(300, 500));
        JScrollPane previewSampleScrollPane = new JScrollPane(previewSample);
        previewSampleScrollPane.setSize(new Dimension(300, 500));
        previewSampleScrollPane.setMinimumSize(new Dimension(300, 500));
        previewSampleScrollPane.setMaximumSize(new Dimension(300, 500));
        previewSampleScrollPane.setPreferredSize(new Dimension(300, 500));

        SimpleAttributeSet redText = new SimpleAttributeSet();
        StyleConstants.setForeground(redText, Color.red);

        inputFileChooser = new JTextField(30);
        orthPathChooser = new JTextField(10);
        orthPathChooser.setText(orthPath);
        posPathChooser = new JTextField(10);
        posPathChooser.setText(posPath);
        outputFileChooser = new JTextField(30);
        dictFileChooser = new JTextField(30);
        ccfFileChooser = new JTextField(30);
        ccfFileChooser.setText("lang/german.ccf");
        preservedPOSChooser = new JTextField(30);
        noMorphPOSChooser = new JTextField(30);
        occPerWordChooser = new JTextField(10);
        occPerWordChooser.setText(Double.toString(occPerWord));
        minOccursChooser = new JTextField(10);
        minOccursChooser.setText(Integer.toString(minOccurs));

        inputFileChanger = new JButton("...");
        inputFileChanger.setActionCommand("Change Input File");
        inputFileChanger.addActionListener(this);
        orthPathChanger = new JButton("Apply");
        orthPathChanger.setActionCommand("Change Orth Path");
        orthPathChanger.addActionListener(this);
        posPathChanger = new JButton("Apply");
        posPathChanger.setActionCommand("Change Pos Path");
        posPathChanger.addActionListener(this);
        outputFileChanger = new JButton("...");
        outputFileChanger.setActionCommand("Change Output File");
        outputFileChanger.addActionListener(this);
        dictFileChanger = new JButton("...");
        dictFileChanger.setActionCommand("Change Dict File");
        dictFileChanger.addActionListener(this);
        ccfFileChanger = new JButton("...");
        ccfFileChanger.setActionCommand("Change CCF File");
        ccfFileChanger.addActionListener(this);
        preservedPOSChanger = new JButton("Apply");
        preservedPOSChanger.setActionCommand("Change preserved POS");
        preservedPOSChanger.addActionListener(this);
        noMorphPOSChanger = new JButton("Apply");
        noMorphPOSChanger.setActionCommand("Change no morph POS");
        noMorphPOSChanger.addActionListener(this);
        occPerWordChanger = new JButton("Apply");
        occPerWordChanger.setActionCommand("Change occ per word");
        occPerWordChanger.addActionListener(this);
        minOccursChanger = new JButton("Apply");
        minOccursChanger.setActionCommand("Change min occurs");
        minOccursChanger.addActionListener(this);

        maskingModeLabel = new JLabel("Masking Mode: Full Strip");
        levelChanger = new JSlider(JSlider.HORIZONTAL, 0, maskingMode, 4);
        levelChanger.setMinorTickSpacing(1);
        levelChanger.setMajorTickSpacing(1);
        levelChanger.setSnapToTicks(true);
        levelChanger.setPaintTicks(true);
        levelChanger.addChangeListener(this);

        startButton = new JButton("--START-->");
        startButton.addActionListener(this);
        
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);

        errorMessages = new JLabel(" ");
        errorMessages.setForeground(Color.RED);
        errorMessages.setHorizontalAlignment(SwingConstants.CENTER);

        logText = new JTextArea("Ready. Please specify input and output file...");

             
        JScrollPane logTextScrollPane = new JScrollPane(logText);
        logTextScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        logTextScrollPane.setSize(300, 500);
        logTextScrollPane.setMinimumSize(new Dimension(300, 500));
        logTextScrollPane.setMaximumSize(new Dimension(650, 500));
        logTextScrollPane.setPreferredSize(new Dimension(500, 500));
        
        JLabel fileSampleLabel = new JLabel("Input file sample:");
        JLabel outputSampleLabel = new JLabel("Output file sample:");

        inputFileLabel = new JLabel("Input File:");
        orthPathLabel = new JLabel("Orth Path:");
        posPathLabel = new JLabel("POS Path:");
        outputFileLabel = new JLabel("Output File:");
        dictFileLabel = new JLabel("Dictionary File:");
        ccfFileLabel = new JLabel("CCF File:");
        preservedPOSLabel = new JLabel("Preserved POS:");
        noMorphPOSLabel = new JLabel("No Morph POS:");
        occPerWordLabel = new JLabel("Occ./Word:");
        minOccursLabel = new JLabel("Min. Occurs");

        fileSample.setText("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
        previewSample.setText("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

        JPanel inputFilePanel = new JPanel();
        inputFilePanel.setLayout(new FlowLayout());
        inputFilePanel.add(inputFileLabel);
        inputFilePanel.add(inputFileChooser);
        inputFilePanel.add(inputFileChanger);

        JPanel xPathsPanel = new JPanel();
        xPathsPanel.setLayout(new FlowLayout());
        xPathsPanel.add(orthPathLabel);
        xPathsPanel.add(orthPathChooser);
        xPathsPanel.add(orthPathChanger);
        xPathsPanel.add(posPathLabel);
        xPathsPanel.add(posPathChooser);
        xPathsPanel.add(posPathChanger);

        JPanel outputFilePanel = new JPanel();
        outputFilePanel.setLayout(new FlowLayout());
        outputFilePanel.add(outputFileLabel);
        outputFilePanel.add(outputFileChooser);
        outputFilePanel.add(outputFileChanger);

        JPanel dictFilePanel = new JPanel();
        dictFilePanel.setLayout(new FlowLayout());
        dictFilePanel.add(dictFileLabel);
        dictFilePanel.add(dictFileChooser);
        dictFilePanel.add(dictFileChanger);
        dictFilePanel.setEnabled(false);
        
        JPanel ccfFilePanel = new JPanel();
        ccfFilePanel.setLayout(new FlowLayout());
        ccfFilePanel.add(ccfFileLabel);
        ccfFilePanel.add(ccfFileChooser);
        ccfFilePanel.add(ccfFileChanger);
        ccfFilePanel.setEnabled(false);
        
        JPanel preservedPOSPanel = new JPanel();
        preservedPOSPanel.setLayout(new FlowLayout());
        preservedPOSPanel.add(preservedPOSLabel);
        preservedPOSPanel.add(preservedPOSChooser);
        preservedPOSPanel.add(preservedPOSChanger);
        
        JPanel noMorphPOSPanel = new JPanel();
        noMorphPOSPanel.setLayout(new FlowLayout());
        noMorphPOSPanel.add(noMorphPOSLabel);
        noMorphPOSPanel.add(noMorphPOSChooser);
        noMorphPOSPanel.add(noMorphPOSChanger);

        JPanel maskingOptionsPanel = new JPanel();
        maskingOptionsPanel.setLayout(new FlowLayout());
        maskingOptionsPanel.add(occPerWordLabel);
        maskingOptionsPanel.add(occPerWordChooser);
        maskingOptionsPanel.add(occPerWordChanger);
        maskingOptionsPanel.add(minOccursLabel);
        maskingOptionsPanel.add(minOccursChooser);
        maskingOptionsPanel.add(minOccursChanger);      

        JPanel maskingPanel = new JPanel();
        maskingPanel
                .setLayout(new BoxLayout(maskingPanel, BoxLayout.PAGE_AXIS));
        maskingPanel.add(maskingModeLabel);
        maskingPanel.add(levelChanger);
        maskingPanel.add(maskingOptionsPanel);
        
        JPanel workingButtons = new JPanel();
        workingButtons.add(startButton);
        workingButtons.add(cancelButton);

        JPanel workingPanel = new JPanel();
        workingPanel
                .setLayout(new BoxLayout(workingPanel, BoxLayout.PAGE_AXIS));
        workingPanel.add(workingButtons);
        workingPanel.add(errorMessages);
        workingPanel.add(logTextScrollPane);

        JPanel leftSide = new JPanel();
        leftSide.setLayout(new BorderLayout());
        leftSide.add(fileSampleLabel, BorderLayout.NORTH);
        leftSide.add(fileSampleScrollPane, BorderLayout.WEST);

        JPanel middlePanel = new JPanel();
        middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.PAGE_AXIS));
        middlePanel.add(inputFilePanel);
        middlePanel.add(xPathsPanel);
        middlePanel.add(outputFilePanel);
        middlePanel.add(dictFilePanel);
        middlePanel.add(maskingPanel);
        middlePanel.add(ccfFilePanel);
        middlePanel.add(preservedPOSPanel);
        middlePanel.add(noMorphPOSPanel);
        middlePanel.add(maskingOptionsPanel);
        middlePanel.add(workingPanel);

        JPanel rightSide = new JPanel();
        rightSide.setLayout(new BorderLayout());
        rightSide.add(outputSampleLabel, BorderLayout.NORTH);
        rightSide.add(previewSampleScrollPane, BorderLayout.EAST);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(leftSide, BorderLayout.WEST);
        getContentPane().add(middlePanel, BorderLayout.CENTER);
        getContentPane().add(rightSide, BorderLayout.EAST);

        updateActive();
        
        //if the window is closed, stop the backend as well
        addWindowListener(new WindowAdapter() 
        {
            public void windowClosing(WindowEvent e) 
            {
                if (guiThread != null) guiThread.stop();
                if (workerThread != null) workerThread.stop();
                System.exit(0);
            }
        });
    }

    public void actionPerformed(ActionEvent e)
    {
        //most changes in parameters will lead to recomputation of the preview
        String s = e.getActionCommand();
        if (s.equals("Change Input File"))
        {
            JFileChooser chooser = new JFileChooser(new File("."));
            chooser.showOpenDialog(this);
            inputFile = chooser.getSelectedFile();
            inputFileName = inputFile.getAbsolutePath();
            inputFileChooser.setText(inputFileName);
            //print a file sample of 5000 characters to the left text pane
            try
            {
                Scanner file = new Scanner(inputFile);
                String sample = "";
                while (sample.length() < 5000)
                {
                    sample += file.nextLine() + "\n";
                }
                file.close();
                fileSample.setText(convHTML(sample));

                updatePreview();
            } catch (IOException ex)
            {
                errorMessages.setText("Unable to open file " + inputFileName);
            }
        } else if (s.equals("Change Orth Path"))
        {
            if (orthPathChooser.getText().equals(""))
            {
                errorMessages.setText("Error: You MUST specify an orth path!");
                orthPathChooser.setText(orthPath);
            }
            else
            {
                orthPath = orthPathChooser.getText();
                updatePreview();
            }
        } else if (s.equals("Change Pos Path"))
        {
            posPath = posPathChooser.getText();
            updatePreview();
        } else if (s.equals("Change Output File"))
        {
            JFileChooser chooser = new JFileChooser(new File("."));
            chooser.showSaveDialog(this);
            outputFile = chooser.getSelectedFile();
            outputFileName = outputFile.getAbsolutePath();
            outputFileChooser.setText(outputFileName);
        } else if (s.equals("Change Dict File"))
        {
            JFileChooser chooser = new JFileChooser(new File("."));
            chooser.showSaveDialog(this);
            dictFile = chooser.getSelectedFile();
            dictFileName = dictFile.getAbsolutePath();
            dictFileChooser.setText(dictFileName);
        } else if (s.equals("Change CCF File"))
        {
            JFileChooser chooser = new JFileChooser(new File("."));
            chooser.showOpenDialog(this);
            File ccfFile = chooser.getSelectedFile();
            ccfFileName = ccfFile.getAbsolutePath();
            ccfFileChooser.setText(ccfFileName);
        } else if (s.equals("Change preserved POS"))
        {
            String i = preservedPOSChooser.getText();
            Scanner p = new Scanner(i);
            HashSet<String> prPOS = new HashSet<String>();
            while (p.hasNext())
            {
                prPOS.add(p.next());
            }
            preservedPOS = prPOS;
            updatePreview();
        } else if (s.equals("Change no morph POS"))
        {
            String i = noMorphPOSChooser.getText();
            Scanner p = new Scanner(i);
            HashSet<String> nMPOS = new HashSet<String>();
            while (p.hasNext())
            {
                nMPOS.add(p.next());
            }
            noMorphPOS = nMPOS;
            updatePreview();
        } else if (s.equals("Change occ per word"))
        {
            //this parameter must stay within the range ]0,1[
            Double d = Double.parseDouble(occPerWordChooser.getText());
            if (d < 0 || d > 1)
            {
                occPerWordChooser.setText(Double.toString(occPerWord));
                errorMessages.setText(d + " invalid: Occ./Word must be a double value between 0 and 1!");
            }
            else
            {
                errorMessages.setText("");
                occPerWord = d;
                updatePreview();
            }
        } else if (s.equals("Change min occurs"))
        {
            Integer i = Integer.parseInt(minOccursChooser.getText());
            if (i < 0)
            {
                minOccursChooser.setText(Integer.toString(minOccurs));
                errorMessages.setText(i + " invalid: Min Occurs must be an integer value above 0!");
            }
            else
            {
                errorMessages.setText("");
                minOccurs = i;
                updatePreview();
            }
        } 
        else if (s.equals("--START-->"))
        {
            setToWorkingMode();
            logText.append("Starting masking operation...\n\n");
            //initialize the message buffers
            Drop logDrop = new Drop();
            Drop errorDrop = new Drop();
            //initialize the thread that handles communication between GUI and backend
            guiThread = new Thread(new GUIUpdateThread(this, logDrop));
            guiThread.start();
            //initialize the thread that handles errors smoothly
            errorThread = new Thread(new ErrorCatcherThread(this, errorDrop));
            errorThread.start();
            //initialize the thread that starts and controls the backend
            workerThread = new Thread(new WorkerThread(logDrop, errorDrop));
            workerThread.start();
        }
        else if (s.equals("Cancel"))
        {
            //force the backend to stop
            workerThread.stop();
            //deactivate the communication threads
            errorThread.stop();
            guiThread.stop();
            logText.setText(logText.getText() + "Action cancelled by user\n");
            logText.setCaretPosition(logText.getDocument().getLength());
            errorMessages.setText("Cancelled by user");
            setToChangeMode();
            repaint();     
        }
    }

    public void stateChanged(ChangeEvent e)
    {
        if (e.getSource() == levelChanger)
        {
            maskingMode = levelChanger.getValue();
            updateModeDisplay();
        }
    }

    /**
     * update the mode display after the slider has been moved
     */
    public void updateModeDisplay()
    {
        switch (maskingMode)
        {
            case 0:
                maskingModeLabel.setText("Masking mode: No masking");
                break;
            case 1:
                maskingModeLabel
                        .setText("Masking mode: Masking with dictionary + keep grammar");
                break;
            case 2:
                maskingModeLabel
                        .setText("Masking mode: Masking with dictionary");
                break;
            case 3:
                maskingModeLabel.setText("Masking mode: Replace with [Xx9]");
                break;
            case 4:
                maskingModeLabel.setText("Masking mode: Full Strip");
                break;
        }
        updatePreview();
        updateActive();
    }

    /**
     * mark the content currently matched with colors;
     * compute a preview (doing a test run of the backend) and display it
     */
    public void updatePreview()
    {
        logText.append("Updating preview...\n\n");
        SAXParserFactory factory = SAXParserFactory.newInstance();
        SAXParser saxParser = null;
        try
        {
            saxParser = factory.newSAXParser();
        } catch (Throwable t)
        {
            t.printStackTrace();
        }
        DicSampleExtractor extractor = new DicSampleExtractor(orthPath,
                posPath, preservedPOS, false, ccfFileName);
        if (maskingMode == 1 || maskingMode == 2)
        {
            try
            {
                // extract the dictionary from the input file
                saxParser.parse(inputFile, extractor);
            } catch (SAXException e)
            {

            } catch (Throwable t)
            {
                t.printStackTrace();
            }
            // reestablish morphology if in mode 4; parameters will determine
            // degree of masking
            if (maskingMode == 1)
            {
                extractor.retainMorphology(occPerWord, minOccurs, noMorphPOS);
            }
        }
        switch (maskingMode)
        {
            case 0:
            {
                StringBuffer origDisp = new StringBuffer();
                StringBuffer previewDisp = new StringBuffer();
                ReplaceMarker rep = new ReplaceMarker(origDisp, previewDisp, 4,
                        orthPath, posPath, null);
                //hand on the sample dictionary to the replace marker
                rep.setConvDict(extractor.getConvDict());
                if (inputFile != null && inputFile.canRead())
                {
                    try
                    {
                        // now let the second parser replace the orth elements
                        // by means of the dictionary
                        saxParser.parse(inputFile, rep);
                    } catch (SAXException e)
                    {
                    } catch (Throwable t)
                    {
                        t.printStackTrace();
                    }
                }
                String orig = convHTML(origDisp.toString());
                fileSample.setText(orig);
                previewSample.setText(orig);
                break;
            }
            case 1:
            {
                StringBuffer origDisp = new StringBuffer();
                StringBuffer previewDisp = new StringBuffer();
                ReplaceMarker rep = new ReplaceMarker(origDisp, previewDisp, 4,
                        orthPath, posPath, null);
                // hand on the sample dictionary to the replace marker
                rep.setConvDict(extractor.getConvDict());
                if (inputFile != null && inputFile.canRead())
                {
                    try
                    {
                        // now let the second parser replace the orth elements
                        // by means of the dictionary
                        saxParser.parse(inputFile, rep);
                    } catch (SAXException e)
                    {
                    } catch (Throwable t)
                    {
                        t.printStackTrace();
                    }
                }
                String orig = convHTML(origDisp.toString());
                String preview = convHTML(previewDisp.toString());
                System.out.println("here!");
                fileSample.setText(orig);
                previewSample.setText(preview);
                break;
            }
            case 2:
            {
                StringBuffer origDisp = new StringBuffer();
                StringBuffer previewDisp = new StringBuffer();
                ReplaceMarker rep = new ReplaceMarker(origDisp, previewDisp, 3,
                        orthPath, posPath, null);
                // hand on the sample dictionary to the replace marker
                rep.setConvDict(extractor.getConvDict());
                if (inputFile != null && inputFile.canRead())
                {
                    try
                    {
                        // now let the second parser replace the orth elements
                        // by means of the dictionary
                        saxParser.parse(inputFile, rep);
                    } catch (SAXException e)
                    {
                    } catch (Throwable t)
                    {
                        t.printStackTrace();
                    }
                }
                String orig = convHTML(origDisp.toString());
                String preview = convHTML(previewDisp.toString());
                fileSample.setText(orig);
                previewSample.setText(preview);
                break;
            }
            case 3:
            {
                StringBuffer origDisp = new StringBuffer();
                StringBuffer previewDisp = new StringBuffer();
                ReplaceMarker rep = new ReplaceMarker(origDisp, previewDisp, 2,
                        orthPath, posPath, null);
                if (inputFile != null && inputFile.canRead())
                {
                    try
                    {
                        // now let the second parser replace the orth elements
                        // by means of the dictionary
                        saxParser.parse(inputFile, rep);
                    } catch (SAXException e)
                    {
                    } catch (Throwable t)
                    {
                        t.printStackTrace();
                    }
                }
                String orig = convHTML(origDisp.toString());
                String preview = convHTML(previewDisp.toString());
                fileSample.setText(orig);
                previewSample.setText(preview);
                break;
            }
            case 4:
            {
                StringBuffer origDisp = new StringBuffer();
                StringBuffer previewDisp = new StringBuffer();
                ReplaceMarker rep = new ReplaceMarker(origDisp, previewDisp, 0,
                        orthPath, posPath, null);
                if (inputFile != null && inputFile.canRead())
                {
                    try
                    {
                        // now let the second parser replace the orth elements
                        // by means of the dictionary
                        saxParser.parse(inputFile, rep);
                    } catch (SAXException e)
                    {
                    } catch (Throwable t)
                    {
                        t.printStackTrace();
                    }
                }
                String orig = convHTML(origDisp.toString());
                String preview = convHTML(previewDisp.toString());
                fileSample.setText(orig);
                previewSample.setText(preview);
                break;
            }
        }
        fileSample.setCaretPosition(0);
        previewSample.setCaretPosition(0);
    }
    
    /**
     * deactivate all the GUI elements and activate the Cancel button
     */
    public void setToWorkingMode()
    {
        startButton.setEnabled(false);
        cancelButton.setEnabled(true);
        
        inputFileChooser.setEnabled(false);
        orthPathChooser.setEnabled(false);
        posPathChooser.setEnabled(false);
        outputFileChooser.setEnabled(false);
        dictFileChooser.setEnabled(false);
        preservedPOSChooser.setEnabled(false);   
        noMorphPOSChooser.setEnabled(false);
        occPerWordChooser.setEnabled(false);
        minOccursChooser.setEnabled(false);

        inputFileChanger.setEnabled(false);
        orthPathChanger.setEnabled(false);
        posPathChanger.setEnabled(false);
        outputFileChanger.setEnabled(false);
        dictFileChanger.setEnabled(false);
        preservedPOSChanger.setEnabled(false);
        noMorphPOSChanger.setEnabled(false);
        occPerWordChanger.setEnabled(false);
        minOccursChanger.setEnabled(false);

        inputFileLabel.setEnabled(false);
        orthPathLabel.setEnabled(false);
        posPathLabel.setEnabled(false);
        outputFileLabel.setEnabled(false);
        maskingModeLabel.setEnabled(false);
        dictFileLabel.setEnabled(false);
        preservedPOSLabel.setEnabled(false);
        noMorphPOSLabel.setEnabled(false);
        occPerWordLabel.setEnabled(false);
        minOccursLabel.setEnabled(false);

        levelChanger.setEnabled(false);
    }
  
    /**
     * reactivate all the GUI elements and disable the Cancel button
     */
    public void setToChangeMode()
    {
        startButton.setEnabled(true);
        
        inputFileChooser.setEnabled(true);
        orthPathChooser.setEnabled(true);
        posPathChooser.setEnabled(true);
        outputFileChooser.setEnabled(true);
        dictFileChooser.setEnabled(true);
        preservedPOSChooser.setEnabled(true);   
        noMorphPOSChooser.setEnabled(true);
        occPerWordChooser.setEnabled(true);
        minOccursChooser.setEnabled(true);

        inputFileChanger.setEnabled(true);
        orthPathChanger.setEnabled(true);
        posPathChanger.setEnabled(true);
        outputFileChanger.setEnabled(true);
        dictFileChanger.setEnabled(true);
        preservedPOSChanger.setEnabled(true);
        noMorphPOSChanger.setEnabled(true);
        occPerWordChanger.setEnabled(true);
        minOccursChanger.setEnabled(true);

        inputFileLabel.setEnabled(true);
        orthPathLabel.setEnabled(true);
        posPathLabel.setEnabled(true);
        outputFileLabel.setEnabled(true);
        maskingModeLabel.setEnabled(true);
        dictFileLabel.setEnabled(true);
        preservedPOSLabel.setEnabled(true);
        noMorphPOSLabel.setEnabled(true);
        occPerWordLabel.setEnabled(true);
        minOccursLabel.setEnabled(true);

        levelChanger.setEnabled(true);
        
        updateActive();
    }

    /**
     * enable the GUI elements relevant for the current masking mode
     * disable all the other GUI elements
     */
    public void updateActive()
    {
        cancelButton.setEnabled(false);
        if (maskingMode == 1)
        {
            noMorphPOSChooser.setEnabled(true);
            occPerWordChooser.setEnabled(true);
            minOccursChooser.setEnabled(true);

            noMorphPOSChanger.setEnabled(true);
            occPerWordChanger.setEnabled(true);
            minOccursChanger.setEnabled(true);

            noMorphPOSLabel.setEnabled(true);
            occPerWordLabel.setEnabled(true);
            minOccursLabel.setEnabled(true);
        }
        else
        {
            noMorphPOSChooser.setEnabled(false);
            occPerWordChooser.setEnabled(false);
            minOccursChooser.setEnabled(false);

            noMorphPOSChanger.setEnabled(false);
            occPerWordChanger.setEnabled(false);
            minOccursChanger.setEnabled(false);

            noMorphPOSLabel.setEnabled(false);
            occPerWordLabel.setEnabled(false);
            minOccursLabel.setEnabled(false);
        }
        switch (maskingMode)
        {
            case 0:
                dictFileLabel.setEnabled(false);
                dictFileChooser.setEnabled(false);
                dictFileChanger.setEnabled(false);
                break;
            case 1:
                dictFileLabel.setEnabled(true);
                dictFileChooser.setEnabled(true);
                dictFileChanger.setEnabled(true);
                break;
            case 2:
                dictFileLabel.setEnabled(true);
                dictFileChooser.setEnabled(true);
                dictFileChanger.setEnabled(true);
                break;
            case 3:
                dictFileLabel.setEnabled(false);
                dictFileChooser.setEnabled(false);
                dictFileChanger.setEnabled(false);
                break;
            case 4:
                dictFileLabel.setEnabled(false);
                dictFileChooser.setEnabled(false);
                dictFileChanger.setEnabled(false);
                break;
        }
    }

    /**
     * escape all HTML special characters for outputting XML using HTML
     * @param toConvert - an XML String
     * @return - an escaped version ready for use in HTML documents
     */
    public String convHTML(String toConvert)
    {
        toConvert = toConvert.replaceAll("<", "&lt;");
        toConvert = toConvert.replaceAll(">", "&gt;");
        toConvert = toConvert.replaceAll("\n", "<br>");
        toConvert = toConvert.replaceAll("\\[\\[", "<");
        toConvert = toConvert.replaceAll("\\]\\]", ">");
        return toConvert;
    }

    public static void main(String[] args)
    {
        CorpusMaskerGUI mainWindow = new CorpusMaskerGUI();
        mainWindow.setSize(1280, 1024);
        mainWindow.setLocation(0, 0);
        mainWindow.setTitle("Corpus Masker");
        mainWindow.setVisible(true);
    }
    
    /**
     * This thread starts the backend and executes the masking task specified by the user via the interface 
     */
    class WorkerThread implements Runnable
    {
        private Drop logDrop;
        private Drop errorDrop;
        
        /**
         * the worker thread has to know what to do with message outputs from the backend
         * @param myLog - a Drop buffer to handle logging output
         * @param myErrorDrop- a Drop buffer to handle error output
         */
        public WorkerThread(Drop myLog, Drop myErrorDrop)
        {
            logDrop = myLog;
            errorDrop = myErrorDrop;
        }
        
        /**
         * let the task be processed
         */
        public void run()
        {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = null;
            try
            {
                saxParser = factory.newSAXParser();
            } catch (Throwable t)
            {
                t.printStackTrace();
            }
            DicExtractor extractor = new DicExtractor(orthPath, posPath, preservedPOS, false, logDrop, ccfFileName);
            if (maskingMode == 1 || maskingMode == 2)
            {
                try
                {
                    // extract the dictionary from the input file
                    saxParser.parse(inputFile, extractor);
                } 
                catch (SAXException f)
                {

                } 
                catch (NullPointerException e)
                {
                    logDrop.put("No input file specified... Aborting"); 
                    errorDrop.put("No input file specified!");  
                }
                catch (IOException e)
                {
                    logDrop.put("Failed to read from input file: " + dictFile.getName());
                    errorDrop.put("Error reading from input file: " + dictFile.getName());
                }
                catch (Throwable t)
                {
                    t.printStackTrace();
                }
                // reestablish morphology if in mode 4; parameters will
                // determine degree of masking
                if (maskingMode == 1)
                {
                    extractor.retainMorphology(occPerWord, minOccurs, noMorphPOS);
                }
                try
                {
                    extractor.getConvDict().writeToFile(dictFile);
                    logDrop.put("Dictionary file successfully written: " + dictFile.getName());
                }
                catch (NullPointerException e)
                {
                    logDrop.put("No dictionary file specified... Aborting");
                    errorDrop.put("No dict file specified!");
          
                }
                catch (IOException e)
                {
                    logDrop.put("Failed to write to dictionary file: " + dictFile.getName());
                    errorDrop.put("Error writing to dict file: " + dictFile.getName());
                }
            } 
            try
            {
                Replacer replacer = new Replacer(outputFile, 1, orthPath, posPath, logDrop);
                replacer.setConvDict(extractor.getConvDict());
                switch (maskingMode)
                {    
                    case 1:
                    {
                        replacer.mode = 4;
                        break;
                    }
                    case 2:
                    {
                        replacer.mode = 3;
                        break;
                    }
                    case 3:
                    {
                        replacer.mode = 2;
                        break;
                    }
                    case 4:
                    {
                        replacer.mode = 1;
                        break;
                    }
                }
                // now let the second parser replace the orth
                // elements by means of the dictionary
                saxParser.parse(inputFile, replacer);
            } 
            catch (SAXException f)
            {
            } 
            catch (NullPointerException e)
            {
                logDrop.put("No output file specified... Aborting"); 
                errorDrop.put("No output file specified!");  
            }
            catch (IOException e)
            {
                logDrop.put("Failed to write to output file: " + dictFile.getName());
                errorDrop.put("Error writing to output file: " + dictFile.getName());
            }
            catch (Throwable t)
            {
                t.printStackTrace();
            }
            logDrop.put("DONE");
        }
    }
    
    /**
     * This thread handles the communication between GUI and backend during processing
     */
    class GUIUpdateThread implements Runnable
    {
        private CorpusMaskerGUI gui;
        private Drop drop;
        
        /**
         * needs a GUI to control and a Drop to get the instructions from
         * @param myGUI - the GUI to control
         * @param theDrop - where to get the messages from
         */
        public GUIUpdateThread(CorpusMaskerGUI myGUI, Drop theDrop)
        {
            gui = myGUI;
            drop = theDrop;
        }
        
        /**
         * start updating the GUI with the information from the message buffer
         */
        public void run() 
        {
            gui.setToWorkingMode();
            for (String message = drop.take(); ! message.equals("DONE"); message = drop.take()) 
            {
                gui.logText.setText(gui.logText.getText() + message + "\n");
                gui.logText.setCaretPosition(logText.getDocument().getLength());
                gui.repaint();
                //update log window 10 times a second
                try 
                {
                    Thread.sleep(100);
                } 
                catch (InterruptedException e) {}
            }
            gui.setToChangeMode();
        }
    }
    
    /**
     * This thread handles the error messaging between GUI and backend during processing
     */   
    class ErrorCatcherThread implements Runnable
    {
        private CorpusMaskerGUI gui;
        private Drop drop;
    
        /**
         * needs a GUI to communicate with and a Drop to get the errors from
         * @param myGUI - the GUI to communicate with
         * @param theDrop - where to get the errors from
         */
        public ErrorCatcherThread(CorpusMaskerGUI myGUI, Drop theDrop)
        {
            gui = myGUI;
            drop = theDrop;
        }
       
        /**
         * start informing the GUI about errors from the buffer
         */
        public void run() 
        {
            gui.errorMessages.setText(" ");
            for (String message = drop.take(); ! message.equals("DONE"); message = drop.take()) 
            {
                if (!message.equals(""))
                {
                    gui.errorMessages.setText(message);
                    gui.repaint();
                    guiThread.stop();
                    workerThread.stop();
                    gui.setToChangeMode();
                    break;
                }
                //update log window 10 times a second
                try 
                {
                    Thread.sleep(100);
                } 
                catch (InterruptedException e) {}
            }
        }
    }
    
}
